import {Injectable} from '@angular/core';
import {HttpClient} from '@angular/common/http';
import {Observable} from "rxjs";
import {Session, User} from "../login/utils/helper.class";
import {map} from "rxjs/operators";
import {properties} from "../../../environments/environment";
import {StringUtils} from "../utils/string-utils.class";
import {CustomOptions} from "./servicesUtils/customOptions.class";
import {AdvancedAsyncSubject} from "../utils/AdvancedAsyncSubject";

@Injectable({
  providedIn: 'root'
})
export class UserManagementService {
  private readonly getUserInfoSubject: AdvancedAsyncSubject<User> = new AdvancedAsyncSubject<User>();
  public fixRedirectURL: string = null;
  private redirectUrl: string = null;
  private subscription;
  private readonly routerSubscription;
  
  constructor(private http: HttpClient) {
    this.updateUserInfo();
  }
  
  clearSubscriptions() {
    if (this.subscription) {
      this.subscription.unsubscribe();
    }
    if (this.routerSubscription) {
      this.routerSubscription.unsubscribe();
    }
  }
  
  public get user(): User {
    return this.getUserInfoSubject.getValue();
  }
  
  public getUserInfo(): Observable<User> {
    return this.getUserInfoSubject.asObservable();
  }
  
  public updateUserInfo(resolve: Function = null) {
    this.subscription = this.http.get<User>(properties.userInfoUrl, CustomOptions.registryOptions()).pipe(map(userInfo => {
      return new User(userInfo);
    })).subscribe(user => {
      this.getUserInfoSubject.next(user);
      if (resolve) {
        resolve();
      }
    }, error => {
      this.getUserInfoSubject.next(null);
      if (resolve) {
        resolve();
      }
    });
  }

  public setRedirectUrl(url: string = null) {
    if (url) {
      let parts = url.split('?');
      let path = properties.baseLink + parts[0];
      let params = null;
      if (parts.length == 2) {
        params = parts[1];
      }
      let hash = path.indexOf("#");
      let fragment = (hash !== -1) ? path.slice(hash + 1) : null;
      if (fragment) {
        path = path.slice(0, hash);
      } else {
        fragment = "";
      }
      if (!path.includes('/reload')) {
        Session.setReloadUrl(location.protocol + "//" + location.host, path, params, fragment);
      }
      this.redirectUrl = StringUtils.URIEncode(location.protocol + "//" + location.host + this.fixRedirectURL);
    } else {
      this.redirectUrl = StringUtils.URIEncode(location.href);
      Session.setReloadUrl(location.protocol + "//" + location.host, location.pathname, location.search, location.hash);
    }
  }
  
  public login() {
    if (this.fixRedirectURL) {
      this.setRedirectUrl(this.fixRedirectURL);
    } else {
      this.setRedirectUrl();
    }
    window.location.href = properties.loginUrl + "?redirect=" + this.redirectUrl;
  }
  
  public logout() {
    this.setRedirectUrl();
    Session.removeUser();
    window.location.href = properties.logoutUrl + "?redirect=" + this.redirectUrl;
  }
}
